/*
    Fichier filtresujet.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme dtruit les fichiers mail comportant une chaine de
    caractres particulire dans le sujet.

    La liste des chaines de caractres interdites est mmorise dans
    un fichier

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "pop.h"
#include "testchamp.h"
#include "trtentete.h"


#define nbmax_sujet   500     // nombre max de sujets interdits


/* prototypes */
int testsujet (int numes);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *listesujet [nbmax_sujet]; // mmorise les sujets interdits
int  sz_listesujet;     // nombre de sujets mmoriss dans listesujet
char bufSubject [80];   // buffer contenant le sujet du mail
FILE *ftrace;           // Fichier contenant les sujets des mail refuss
char datecour [10];     // Date courante (pour le fichier des mails refuss)


/* programme principal */

int main (int nbarg, char *varg[])
{
    char ficonf [80];    // fichier de configuration du systme de mail
    FILE *fconf;         // descripteur du fichier de configuration
    char fichliste [90]; // fichier contenant la liste des chaines  tester
    int  numes, nbmes;   // mumro du mail courant et nombre de mails
    char bufw [80];      // buffer d'envoi d'une requte de destruction de mail
    int  conserves = 0;  // nombre de mails conservs
    int  supprimes = 0;  // nombre de mails supprims
    char *ncom;          // nom de la commande que l'on excute (filtresujet)


    // rcupration du nom de l'excutable
    ncom = varg [0];
    nbarg --;
    varg ++;

    // si le programme a t lanc avec l'option -t
    if (nbarg == 3 && strcmp (*varg, "-t") == 0)
    {
        // ouvrir le fichier trace en criture fin de fichier
        ftrace = fopen (varg [1], "a");

        // si nom de fichier correct
        if (ftrace)
            // initialiser la date courante
            initdatecour (datecour);
        else
            // sinon, avertir l'utilisateur
            fprintf (stderr, "Impossible d'crire dans le fichier %s\n",
                                                               varg [1]);

        // sauter les arguments traits
        nbarg -= 2;
        varg += 2;
    }
    // sinon, pas d'utilisation d'un fichier trace
    else
        ftrace = NULL;

    // controle du nombre d'arguments restants
    if (nbarg == 1)
    {
        // rcuprer le nom du fichier de configuration
        strcpy (ficonf, *varg);

        if (ficonf [strlen (ficonf) - 4] != '.')
            strcat (ficonf, ".cfg");

        // ouvrir le fichier de configuration
        fconf = fopen (ficonf, "r");

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // chercher le nom du fichier contenant les sujets interdits
                fgets (fichliste, sizeof (fichliste), fconf);
                strcpy (fichliste + strlen (fichliste) - 1, "/refus_sujet");

                // charger en mmoire les sujets interdits
                sz_listesujet = charge_valchamp (fichliste, listesujet,
                                                 nbmax_sujet, "nbmax_sujet");

                // si la liste n'est pas vide
                if (sz_listesujet)
                {
                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // vrification des diffrents mails
                    for (numes = 1; numes <= nbmes; numes++)
                    {
                        // si le sujet du mail contient une chaine interdite
                        if (testsujet (numes))
                        {
                            // demande de destruction du mail
                            sprintf (bufw, "DELE %d", numes);
                            env_pop (bufw);
                            lire_pop ();
                            supprimes ++;

                            // si utilisation d'un fichier trace
                            if (ftrace)
                                // mmoriser le sujet de ce mail
                                fprintf (ftrace, "%s%s\n", datecour, bufSubject);
                        }
                        else
                            conserves ++;
                    }

                    // dition d'un rcapitulatif
                    printf ("\n%d messages conservs, %d supprims\n",
                                              conserves, supprimes);
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();

                // si des mails ont t supprims
                if (supprimes)
                    // attendre pour enregistrement correct des suppressions
                    // (vite problme si autre filtre appel juste aprs)
                    sleep (2);
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
        else
            fprintf (stderr, "Fichier %s non trouv\n", ficonf);
    }
    else
fprintf (stderr, "Syntaxe : %s [-t fichier_trace] fich_configuration\n", ncom);

    return (0);
}



/* lit l'entte d'un mail et vrifie si son sujet est autoris */

int testsujet (int numes)
{
    char bufw [80];       // buffer d'envoi d'une requte
    int  i;               // simple compteur


    printf ("\rMessage %d", numes);
    fflush (stdout);

    // initialisation
    bufSubject [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de le sujet du mail
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si on a trouv le champ sujet
        if (start ("Subject"))
        {
            // convertir les caractres spciaux de la ligne
            majlignentete ();

            // initialisation
            i = 8;

            // recherche du dbut du sujet
            while (buf_lect [i] == ' ')
                i++;

            // tronquer si ncessaire le sujet (pour viter un plantage)
            if (strlen (buf_lect + i) > sizeof (bufSubject))
                 buf_lect [i + sizeof (bufSubject) - 1] = '\0';

            // mmorisation du sujet
            strcpy (bufSubject, buf_lect + i);
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // vrifier si le mail contient un des sujet de la liste interdite
    return trouve_valchamp (bufSubject, listesujet, sz_listesujet);
}
